-- Nancy Minderman
-- nancy.minderman@ualberta.ca
-- Modified January 8th 2013
-- Changed port mappings from positional or ordered to explicit
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity CI_test is
	port
	(
      -- DRAM ports
      DRAM_CLK		: 	out 		std_logic;
		DRAM_CKE 	: 	out 		std_logic;
      DRAM_ADDR	: 	out 		std_logic_vector(11 DOWNTO 0);
      DRAM_BA_1	: 	buffer 	std_logic;
		DRAM_BA_0 	: 	buffer 	std_logic;
      DRAM_CS_N 	: 	out 		std_logic; 
		DRAM_CAS_N 	: 	out 		std_logic; 
		DRAM_RAS_N 	: 	out 		std_logic; 
		DRAM_WE_N 	: 	out 		std_logic;
      DRAM_DQ 		: 	inout 	std_logic_vector(15 DOWNTO 0);
      DRAM_UDQM 	:	buffer 	std_logic; 
		DRAM_LDQM 	: 	buffer 	std_logic ;

		-- GPIO ports
      GPIO_0		: 	inout			std_logic_vector(35 downto 0);
		GPIO_1		:	inout			std_logic_vector(35 downto 0);
		
      -- CLOCK port
      CLOCK_50 	:	in  		std_logic;
					 
      -- RESET key
      KEY 			:	in  		std_logic_vector(0 downto 0)
      );
end CI_test;


-- Library Clause(s) (optional)
-- Use Clause(s) (optional)

architecture structure of CI_test is

-- A component declaration declares the interface of an entity or
-- a design unit written in another language.  VHDL requires that
-- you declare a component if you do not intend to instantiate
-- an entity directly.  The component need not declare all the
-- generics and ports in the entity.  It may omit generics/ports
-- with default values.

component niosII_system

        port
        (
              -- 1) global signals:
                 signal altpll_0_c0_out : OUT STD_LOGIC;
                 signal altpll_0_c1_out : OUT STD_LOGIC;
                 signal clk_0 : IN STD_LOGIC;
                 signal reset_n : IN STD_LOGIC;

              -- the_altpll_0
                 signal locked_from_the_altpll_0 : OUT STD_LOGIC;
                 signal phasedone_from_the_altpll_0 : OUT STD_LOGIC;

              -- the_sdram_0
                 signal zs_addr_from_the_sdram_0 : OUT STD_LOGIC_VECTOR (11 DOWNTO 0);
                 signal zs_ba_from_the_sdram_0 : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
                 signal zs_cas_n_from_the_sdram_0 : OUT STD_LOGIC;
                 signal zs_cke_from_the_sdram_0 : OUT STD_LOGIC;
                 signal zs_cs_n_from_the_sdram_0 : OUT STD_LOGIC;
                 signal zs_dq_to_and_from_the_sdram_0 : INOUT STD_LOGIC_VECTOR (15 DOWNTO 0);
                 signal zs_dqm_from_the_sdram_0 : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
                 signal zs_ras_n_from_the_sdram_0 : OUT STD_LOGIC;
                 signal zs_we_n_from_the_sdram_0 : OUT STD_LOGIC;
					  
	           -- the_uart_0
                 signal rxd_to_the_uart_0 : IN STD_LOGIC;
                 signal txd_from_the_uart_0 : OUT STD_LOGIC;
					  
				  -- the_uart_1
                 signal rxd_to_the_uart_1 : IN STD_LOGIC;
                 signal txd_from_the_uart_1 : OUT STD_LOGIC

        );
end component;

        signal BA : std_logic_vector (1 downto 0);
        signal DQM : std_logic_vector (1 downto 0);

		  -- I have no use for these signals from the pll so I just
		  -- connect them with signals
		  -- I specifically asked the PLL to omit these signals and yet
		  -- they are generated anyway
		  
        signal pll_c1:std_logic;
        signal pll_locked:std_logic;
        signal pll_phase: std_logic;

begin
		-- I need to make the mapping between the BA and DQM lines of the niosII_system
		-- and the physical DRAM 
        DRAM_BA_1 <= BA(1);
        DRAM_BA_0 <= BA(0);
        DRAM_UDQM <= DQM(1);
        DRAM_LDQM <= DQM(0);

        -- Component Instantiation Statement (optional)
        NIOSII : niosII_system port map ( 
		  altpll_0_c0_out => DRAM_CLK, 
		  altpll_0_c1_out => pll_c1, 
		  locked_from_the_altpll_0 => pll_locked,
		  phasedone_from_the_altpll_0 => pll_phase,
		  zs_addr_from_the_sdram_0 => DRAM_ADDR, 
		  zs_ba_from_the_sdram_0 => BA, 
		  zs_cas_n_from_the_sdram_0 => DRAM_CAS_N, 
		  zs_cke_from_the_sdram_0 => DRAM_CKE, 
		  zs_cs_n_from_the_sdram_0 => DRAM_CS_N,
		  zs_dq_to_and_from_the_sdram_0 => DRAM_DQ, 
		  zs_dqm_from_the_sdram_0 => DQM, 
		  zs_ras_n_from_the_sdram_0 => DRAM_RAS_N, 
		  zs_we_n_from_the_sdram_0 => DRAM_WE_N,
		  clk_0 => CLOCK_50, 
		  reset_n => KEY(0),
		  
		  rxd_to_the_uart_0 => GPIO_0(1),
		  txd_from_the_uart_0 => GPIO_0(3),
		  rxd_to_the_uart_1 => GPIO_0(5),
		  txd_from_the_uart_1 => GPIO_0(7)
		  );
end structure;
		  

